@@ -41,6 +41,8 @@
 
 #include "fs-rtp-conference.h"
 
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "fs-rtp-session.h"
@@ -357,23 +359,13 @@ fs_rtp_conference_init (FsRtpConference *conf,
 static void
 rtpbin_get_sdes (FsRtpConference *self, const gchar *prop, GValue *val)
 {
-  if (g_object_class_find_property (G_OBJECT_GET_CLASS (self->gstrtpbin),
-          "sdes"))
-  {
-    GstStructure *s;
-    const GValue *sval;
-    g_object_get (self->gstrtpbin, "sdes", &s, NULL);
-    sval = gst_structure_get_value (s, prop);
-    if (sval)
-      g_value_copy (gst_structure_get_value (s, prop), val);
-    gst_structure_free (s);
-  }
-  else
-  {
-    gchar *str = g_strdup_printf ("sdes-%s", prop);
-    g_object_get_property (G_OBJECT (self->gstrtpbin), str, val);
-    g_free (str);
-  }
+  GstStructure *s;
+  const GValue *sval;
+  g_object_get (self->gstrtpbin, "sdes", &s, NULL);
+  sval = gst_structure_get_value (s, prop);
+  if (sval)
+    g_value_copy (gst_structure_get_value (s, prop), val);
+  gst_structure_free (s);
 }
 
 static void
@@ -419,21 +411,11 @@ fs_rtp_conference_get_property (GObject *object,
 static void
 rtpbin_set_sdes (FsRtpConference *self, const gchar *prop, const GValue *val)
 {
-  if (g_object_class_find_property (G_OBJECT_GET_CLASS (self->gstrtpbin),
-          "sdes"))
-  {
-    GstStructure *s;
-    g_object_get (self->gstrtpbin, "sdes", &s, NULL);
-    gst_structure_set_value (s, prop, val);
-    g_object_set (self->gstrtpbin, "sdes", s, NULL);
-    gst_structure_free (s);
-  }
-  else
-  {
-    gchar *str = g_strdup_printf ("sdes-%s", prop);
-    g_object_set_property (G_OBJECT (self->gstrtpbin), str, val);
-    g_free (str);
-  }
+  GstStructure *s;
+  g_object_get (self->gstrtpbin, "sdes", &s, NULL);
+  gst_structure_set_value (s, prop, val);
+  g_object_set (self->gstrtpbin, "sdes", s, NULL);
+  gst_structure_free (s);
 }
 
 static void
@@ -907,7 +889,8 @@ fs_codec_to_gst_caps (const FsCodec *codec)
 
   for (item = codec->optional_params;
        item;
-       item = g_list_next (item)) {
+       item = g_list_next (item))
+  {
     FsCodecParameter *param = item->data;
     gchar *lower_name = g_ascii_strdown (param->name, -1);
 
@@ -920,6 +903,31 @@ fs_codec_to_gst_caps (const FsCodec *codec)
     g_free (lower_name);
   }
 
+  for (item = codec->ABI.ABI.feedback_params;
+       item;
+       item = g_list_next (item))
+  {
+    FsFeedbackParameter *param = item->data;
+    gchar *lower_type = g_ascii_strdown (param->type, -1);
+    gchar *rtcpfb_name;
+
+    if (param->subtype[0])
+    {
+      gchar *lower_subt = g_ascii_strdown (param->subtype, -1);
+      rtcpfb_name = g_strdup_printf ("rtcp-fb-%s-%s", lower_type, lower_subt);
+      g_free (lower_subt);
+    }
+    else
+    {
+      rtcpfb_name = g_strdup_printf ("rtcp-fb-%s", lower_type);
+    }
+
+    gst_structure_set (structure,
+        rtcpfb_name, G_TYPE_STRING, param->extra_params, NULL);
+    g_free (lower_type);
+    g_free (rtcpfb_name);
+  }
+
   caps = gst_caps_new_full (structure, NULL);
 
   return caps;
