@@ -420,15 +420,6 @@ fs_rtp_special_sources_remove (
   return changed;
 }
 
-static gboolean
-_source_order_compare_func (gconstpointer item1,gconstpointer item2)
-{
-  FsRtpSpecialSource *src1 = FS_RTP_SPECIAL_SOURCE_CAST (item1);
-  FsRtpSpecialSource *src2 = FS_RTP_SPECIAL_SOURCE_CAST (item2);
-
-  return src1->order - src2->order;
-}
-
 /**
  * fs_rtp_special_sources_create:
  * @current_extra_sources: A pointer to the #GList returned by previous calls
@@ -508,8 +499,7 @@ fs_rtp_special_sources_create (
       }
       else
       {
-        *extra_sources = g_list_insert_sorted (*extra_sources,
-            obj, _source_order_compare_func);
+        *extra_sources = g_list_prepend (*extra_sources, obj);
         changed = TRUE;
       }
     }
@@ -601,160 +591,6 @@ fs_rtp_special_source_new (FsRtpSpecialSourceClass *klass,
   return NULL;
 }
 
-/**
- * fs_rtp_special_source_send_event:
- * @self: a #FsRtpSpecialSource
- * @event: a upstream #GstEvent to send
- *
- * Sends an upstream event to the source.
- *
- * Returns: %TRUE if the event was delivered succesfully, %FALSE otherwise
- */
-
-static gboolean
-fs_rtp_special_source_send_event (FsRtpSpecialSource *self,
-    GstEvent *event)
-{
-  gboolean ret = FALSE;
-  GstPad *pad;
-
-  pad = gst_element_get_static_pad (self->priv->src, "src");
-
-  if (!pad)
-  {
-    GST_ERROR ("Could not find the source pad on the special source");
-    gst_event_unref (event);
-    return FALSE;
-  }
-
-  ret = gst_pad_send_event (pad, event);
-
-  gst_object_unref (pad);
-
-  return ret;
-}
-
-/**
- * fs_rtp_special_sources_send_event:
- * @current_extra_sources: The #GList of current #FsRtpSpecialSource
- * @event: an upstream #GstEvent
- *
- * This function will try to deliver the events in the specified order to the
- * special sources, it will stop once one source has accepted the event.
- *
- * Returns: %TRUE if a sources accepted the event, %FALSE otherwise
- */
-
-static gboolean
-fs_rtp_special_sources_send_event (GList *current_extra_sources,
-    GstEvent *event)
-{
-  GList *item = NULL;
-
-  if (!event)
-  {
-    GST_ERROR ("Could not make dtmf-event");
-    return FALSE;
-  }
-
-  for (item = g_list_first (current_extra_sources);
-       item;
-       item = g_list_next (item))
-  {
-    FsRtpSpecialSource *source = item->data;
-    gst_event_ref (event);
-    if (fs_rtp_special_source_send_event (source, event))
-    {
-      gst_event_unref (event);
-      return TRUE;
-    }
-  }
-  gst_event_unref (event);
-  return FALSE;
-}
-
-
-gboolean
-fs_rtp_special_sources_start_telephony_event (GList *current_extra_sources,
-      guint8 event,
-      guint8 volume,
-      FsDTMFMethod method)
-{
-  GstStructure *structure = NULL;
-  gchar *method_str;
-
-  structure = gst_structure_new ("dtmf-event",
-      "number", G_TYPE_INT, event,
-      "volume", G_TYPE_INT, volume,
-      "start", G_TYPE_BOOLEAN, TRUE,
-      "type", G_TYPE_INT, 1,
-      NULL);
-
-  if (!structure)
-  {
-    GST_ERROR ("Could not make dtmf-event structure");
-    return FALSE;
-  }
-
-  switch (method)
-  {
-    case FS_DTMF_METHOD_AUTO:
-      method_str = "default";
-      break;
-    case FS_DTMF_METHOD_RTP_RFC4733:
-      method_str="RFC4733";
-      gst_structure_set (structure, "method", G_TYPE_INT, 1, NULL);
-      break;
-    case FS_DTMF_METHOD_IN_BAND:
-      method_str="sound";
-      gst_structure_set (structure, "method", G_TYPE_INT, 2, NULL);
-      break;
-    default:
-      method_str="other";
-  }
-
-  GST_DEBUG ("sending telephony event %d using method=%s",
-      event, method_str);
-
-  return fs_rtp_special_sources_send_event (current_extra_sources,
-      gst_event_new_custom (GST_EVENT_CUSTOM_UPSTREAM, structure));
-}
-
-gboolean
-fs_rtp_special_sources_stop_telephony_event (GList *current_extra_sources,
-    FsDTMFMethod method)
-{
-  GstStructure *structure = NULL;
-  gchar *method_str;
-
-  structure = gst_structure_new ("dtmf-event",
-      "start", G_TYPE_BOOLEAN, FALSE,
-      "type", G_TYPE_INT, 1,
-      NULL);
-
-  switch (method)
-  {
-    case FS_DTMF_METHOD_AUTO:
-      method_str = "default";
-      break;
-    case FS_DTMF_METHOD_RTP_RFC4733:
-      method_str="RFC4733";
-      gst_structure_set (structure, "method", G_TYPE_INT, 1, NULL);
-      break;
-    case FS_DTMF_METHOD_IN_BAND:
-      method_str="sound";
-      gst_structure_set (structure, "method", G_TYPE_INT, 2, NULL);
-      break;
-    default:
-      method_str="unknown (defaulting to auto)";
-  }
-
-  GST_DEBUG ("stopping telephony event using method=%s", method_str);
-
-  return fs_rtp_special_sources_send_event (current_extra_sources,
-      gst_event_new_custom (GST_EVENT_CUSTOM_UPSTREAM, structure));
-}
-
 GList *
 fs_rtp_special_sources_destroy (GList *current_extra_sources)
 {
