@@ -102,11 +102,6 @@ struct _FsRtpSpecialSource
 {
   GObject parent;
 
-  /* This is set by subclasses init or build methods and is used to know
-   * in which order to try events on.
-   */
-  guint order;
-
   FsCodec *codec;
 
   FsRtpSpecialSourcePrivate *priv;
